package vishnu.testvis.sammon;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import vishnu.testvis.visual.*;
import vishnu.testvis.object.*;
import java.util.*;

/** Panel containing the sammon map interface */
public class GraphicalPane extends JPanel
{
    public JPanel picturePanel;
    public ClusterPanel clusterPanel;
    public SammKeywordPanel keywordPanel;
    public HLinkPanel hLinkPanel;
    public Stack dataStack;
    
    public DataManager dataMan=null;
    public Vishnu currentMain;
    public JLabel pictureLabel = new JLabel("Level 0");
    public JButton pictureButton = new JButton("Back");
    public JPanel topPanel = new JPanel();
    private int largeFont = 16;
    private int smallFont = 14;
    private int level = 0;
    
    public GraphicalPane(Vishnu currentMain)
    {
	this.currentMain = currentMain;
	dataMan = currentMain.dataManager;
	
	dataStack=new Stack();
	level=0;
	setName("Sammon Map");
	setLayout(new BorderLayout(10,10));
	setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	clusterPanel = new ClusterPanel(currentMain,this);
	keywordPanel = new SammKeywordPanel(currentMain,this);
	Dimension d = currentMain.getSize();
	hLinkPanel = new HLinkPanel((int)(d.width)/4,d.height,currentMain);
	picturePanel = new JPanel();
	picturePanel.setLayout(new BorderLayout(0,3));
	topPanel.setLayout(new FlowLayout(FlowLayout.LEFT,10,0));
	topPanel.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
	pictureLabel.setFont(new java.awt.Font("Monospaced", 0, largeFont));
	pictureButton.setFont(new java.awt.Font("Monospaced", 0, largeFont));
	topPanel.add(pictureLabel);
	topPanel.add(pictureButton);
	pictureButton.addMouseListener(new java.awt.event.MouseAdapter()
	    {
		public void mouseClicked(MouseEvent e)
		{
		    unDrill();
		}
	    });
	
	pictureButton.setEnabled(false);
	picturePanel.add(topPanel,"North");

	picturePanel.add(clusterPanel,"Center");
	add("East", hLinkPanel.linkPane);
	add("Center", picturePanel);
	add("West", keywordPanel);
	setVisible(true);
    }
    
    public void drillDown(Vector idVect)
    {
	System.out.println("Drilling");

	// save old data manager and create new one with old data
	// and vector of document ids of the selected cluster
	dataStack.push(dataMan);
	dataMan = new DataManager(dataMan.getDataBlock(), idVect, currentMain);
	level++;
	pictureButton.setEnabled(true);
	keywordPanel.clear();
	hLinkPanel.clear();
	pictureLabel.setText("Level "+ level);
    }
    
    public void unDrill()
    {
	if(!dataStack.empty()){
		dataMan = (DataManager)dataStack.pop();
		hLinkPanel.clear();
		keywordPanel.clear();
		level--;
		clusterPanel.repaint();
		pictureLabel.setText("Level "+ level);
	    }
	else level = 0;
	if(level==0)
	    pictureButton.setEnabled(false);
    }
  
    public void update()
    {
	dataMan = currentMain.dataManager;
	dataStack=new Stack(); // dispose of old stack - create new empty one
	level=0;
	clusterPanel.repaint();
	keywordPanel.clear();
	hLinkPanel.clear();
    }
  
    
    public void addNotify()
    {
        Dimension d = getSize();
	//Record the size of the window prior to calling parents addNotify.
        super.addNotify();
	
        if (fComponentsAdjusted)
	    return;
	
	// Adjust components according to the insets
        setSize(getInsets().left + getInsets().right + d.width,
		getInsets().top + getInsets().bottom + d.height);
        Component components[] = getComponents();
        for (int i = 0; i < components.length; i++)
	    {
		Point p = components[i].getLocation();
		p.translate(getInsets().left, getInsets().top);
		components[i].setLocation(p);
	    }
        fComponentsAdjusted = true;
    }
    
    /** Used for addNotify check. */
    protected boolean fComponentsAdjusted = false;
}















