/**
 * @(#)NavigatorPanel.java  12/12/99
 * @author Peter Au
 * All rights reserved.
 */

package vishnu.testvis.treemap.visual;

import vishnu.testvis.visual.Vishnu;
import vishnu.testvis.object.ClusterObj;
import vishnu.testvis.object.DataManager;
import vishnu.testvis.treemap.table.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Vector;


public class NavigatorPanel extends JComponent
{
    private JTable _table;
    private JButton _showAllButton, _refineButton;
    private JEditorPane _display;
    private TMNavigator _outerFrame;
    private ClusterObj _cObj;
//public String _codeBase;

    private RW_DataModel _dataModel;  //-- this is created within this class


    public NavigatorPanel(JTable t, /*JButton b1,*/ JButton b2,
        JEditorPane d, TMNavigator f)
    {
        _table = t;
        /*_showAllButton = b1;
        _showAllButton.setActionCommand("showAllDoc");
        _showAllButton.setToolTipText("show ALL documents within the selected cluster");
        _showAllButton.addActionListener(new ButtonHandler());*/

        _refineButton = b2;
        _refineButton.setActionCommand("refineDoc");
        _refineButton.setToolTipText("show documents with any of the selected related-words");
        _refineButton.addActionListener(new ButtonHandler());

        _display = d;
        _outerFrame = f;

        _display.addHyperlinkListener(new HyperlinkListener() {
                public void hyperlinkUpdate(HyperlinkEvent e)
                {
	            // Handle clicks; ignore mouseovers and other link-related events
	                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
                    {
	            // Get the HREF of the link and display it.
                        String hpresult = e.getDescription();
                        int index = Integer.parseInt(hpresult);
                        System.out.println("hyperlink result: " + index);
                        if (index > 0)
                            _outerFrame._displayFrame.dataManager.fetchDocNo(index);
	                }
                }
            });
        _dataModel = new RW_DataModel();
        _table.setModel(_dataModel);



        _cObj = null;
        unableButtons();
        setupRenderer();
    }

    private void unableButtons()
    {
        //_showAllButton.setEnabled(false);
        _refineButton.setEnabled(false);
    }
    private void enableButtons()
    {
        //_showAllButton.setEnabled(true);
        _refineButton.setEnabled(true);
    }

    public void clear()
    {
        _cObj = null;
        _dataModel.clear();
        _display.setText("");
        unableButtons();
    }



    public void setData(String[] rw, ClusterObj c)
    {
        _cObj = c;
        _dataModel.setData(rw, _cObj);
        enableButtons();
        setupRenderer();
    }


    private void setupRenderer()
    {
        BarRenderer bar = new BarRenderer();
        //-- Have a number painted in the middle of the bar
        bar.setStringPainted(true);
        //-- setup the background color of the bar
        bar.setBackground(_table.getBackground());

        //-- render the last column as a bar
        _table.getColumnModel().getColumn(2).setCellRenderer(bar);
    }
    
    public void showDocuments()
    {
        if( _cObj != null )
        {
            _display.setText(_cObj.formatDocs());
        }
    }
     

    class ButtonHandler implements ActionListener
    {
        public void actionPerformed(ActionEvent event)
        {
            if (_cObj!=null)
            {
                String action = event.getActionCommand();
                if (action.equals("showAllDoc"))
                {
                    _outerFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                    _display.setText(_cObj.formatDocs());
                    _outerFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                }
                else if (action.equals("refineDoc"))
                {
                    _outerFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                    _display.setText(   _dataModel.getRefinedDocs()   );
                    _outerFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                    
                    _outerFrame._displayFrame.log.startsection("FILTERING");
       
                    String trec_id[] = _dataModel.getRefinedTrecIDs();
                    String filter_Words[] = _dataModel.getFilterWords();
                    String s = new String("");
                    for( int i = 0; i < filter_Words.length; i++ )
                    {
                            s += filter_Words[i] + " ";
                    }
      
                   _outerFrame._displayFrame.log.log("FILTER_WORDS",s);
                   _outerFrame._displayFrame.log.log("FILTERED_DOCS",Integer.toString(trec_id.length));
       
                   Vector performance = _outerFrame._displayFrame.queries.getPerformance(_outerFrame._displayFrame.query, trec_id);
                   
                   _outerFrame._displayFrame.log.startsection("PERFORMANCE");
                    
                   if( performance.size() > 0 )
                   {
                        Integer relvTotal = (Integer)performance.elementAt(performance.size()-2);
                        Integer relvRetr = (Integer)performance.elementAt(performance.size()-1);
                        _outerFrame._displayFrame.log.log("RELEVANT_TOTAL",Integer.toString(relvTotal.intValue()));
                        _outerFrame._displayFrame.log.log("RELEVANT_RETRIEVED",Integer.toString(relvRetr.intValue()));
                        for( int i = 0; i < performance.size()-2; i++ )
                        {
                            Integer perf = (Integer)performance.elementAt(i);
                            String str = "PRECISION_" + Integer.toString((i+1)*10);
                            _outerFrame._displayFrame.log.log(str,Integer.toString(perf.intValue()));
                        }
                   }
                   else
                   {
                        _outerFrame._displayFrame.log.log("INFO","No relevance data available");
                   }
                
                    _outerFrame._displayFrame.log.endsection("PERFORMANCE");
                    _outerFrame._displayFrame.log.endsection("FILTERING");
                    
                }
            }
        }
    }  //-- end ButtonHandler



}
