package vishnu.testvis.visual;

import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.awt.Cursor;
import vishnu.testvis.object.*;
import vishnu.datablock.*;


public class CvManyPoints extends Canvas
{
    protected Vishnu displayFrame;
    protected RadVizPanel radVizPanel;
    protected DataManager dataManager;
    protected int [] docs;
    
    /** constant square shape flag */
    static final int SQUARESHAPE = 0;
    /** constant round shape flag */
    static final int ROUNDSHAPE = 1;
    /** constant diamond shape flag */
    static final int DIAMONDSHAPE = 2;
    /** constant maximum keywords allowed */
    static final int ABSMAXKEYWORD = 1000;
    /** constant maximum documents allowed */
    static final int ABSMAXDOC = 1000;
    /** count of the documents */
    protected int documentCount = 1000;
    /** circle array size */
    protected int maxCircleCount = 40;
    /** number of circles */
    protected int circleCount = 40;
    /** number of spokes */
    protected int spokeCount = 16;
    /** radius of the display in fp units */
    protected float r = 10.0F;
    /** locked flag */
    protected boolean isLocked = false;
    /** projected flag */
    protected boolean isProjected = true;
    /** document selected flag */
    protected boolean atDoc = false;
    /** grid flag */
    protected boolean displayGrid = false;
    /** rim flag */
    protected boolean displayRim = true;
    /** doc point shape flag */
    protected int docsShape = SQUARESHAPE;
    /** edge node shape flag */
    protected int nodesShape = ROUNDSHAPE;
    /** colour of nodes */
    protected Color defNodesColor = Color.black;
    /** colour of docs */
    protected Color defDocsColor = Color.orange;
    /** colour of highlighted nodes */
    protected Color defHighlight = Color.blue;
    /** colour of selected documents */
    protected Color defSelect = Color.red;
    /** Radius divided by radians */
    protected float magic = (float)((double)r / (Math.PI / 2.0) );
    /** projection factor */
    protected float factor = 3F;
    /** image object used for double buffering graphics */
    protected Image image=null;
    /** graphics used for double buffering graphics */
    protected Graphics gImage=null;
    /** double buffered image size */
    protected int imgW,imgH;
    /** dimensions of the canvas */
    protected Dimension dim;
    /** the integer location of 0,0 fp axis */
    protected int centerX, centerY;
    /** virtual pixel size and canvas fp dimensions */
    protected float pixelSize, rWidth = 25.0F, rHeight = 25.0F;
    /** array of edge node locations */
    protected Point2D[] points= null;
    /** selected edge node */
    protected Point2D tp =null;
    /** selected edge node index */
    protected int tpIndex = 0;
    /** angle stored for rotating edge nodes */
    protected float lastAngle;
    /** Full 2D linked list of data */
    
    protected Pt2DNode matrix= null;
    /** Array of partial lists for each key word */
    protected Pt2DNode columns[] = null;
    /** The currently selected document if any */
    protected Pt2DNode selectDoc = null;
    /** A class that stores the keywords and whether they are displayed */
    protected KeyWordArray keyWordArray;
    /** Array of linked lists that stores the data to be displayed */
    protected SparseMatrix theDataMatrix;
    /** A class that maps displayed keywords to stored keywords */
    protected MapKeyWords mapping;
    /** A two dimensional array of integers used for clustering */
    //protected int [][] fullMatrix;
    protected float [][] fullMatrix;
    /** array of data to draw target */
    protected int circles[][] = new int[maxCircleCount][4];
    /** array of data to draw rim */
    protected int rim[] = new int[4];
    /** array of data to draw spokes */
    protected int spokes[][] = new int [spokeCount][2];
    /** font for the mouse 'balloon' */
    protected Font balloonFont = null;
    /** metrics of the font for the mouse 'balloon' */
    protected FontMetrics balloonFontMetrics = null;
    /** Flag if drawing needs to just pasted from the buffer or redrawn */
    protected boolean isDirty = true;
    /** Scale the non-projected output if it goes off the page */
    protected float scale=1.0F;
    /** Last scale used when circles calculated (Flat mode only) */
    protected float lastCircleScale = 1.0F;

    /**
     *  Rotate a Point2D around the 0.0,0.0 axis by phi angle (in radians)
     *  @param Point2D around the 0.0,0.0 axis by
     *  @param phi angle (in radians)
     */
    protected void rotatePt(Point2D pt,float phi)
    {
        float c = (float)Math.cos(phi), s = (float)Math.sin(phi),
            r11 = c, r12 = s, r21 = -s, r22 = c;
        float xNew = pt.x * r11 + pt.y * r21,
            yNew = pt.x * r12 + pt.y * r22;
        pt.x = xNew; pt.y = yNew;
	
    }
    /**
    * This is a public gateway for export of the keyWordsArray class
    */
    public KeyWordArray getKeyWords()
    {
        return keyWordArray;
    }


    /**
    *  Fill the integer array circles[circleCount][4] with the correct numbers for
    *  circleCount concentric circles about the 0.0,0.0 axis
    */

    protected void setCircles()
    {
        float  gap =r/4.0F, t = r/4.0F;
        rim[0]=iX(-r); rim[1]=iY(r);
        rim[2]= (int) (2.0F * r / pixelSize);
        rim[3] = rim[2];
        for (int count=0;count < circleCount; count++)
        {
            Point2D pc = new Point2D(-t,0.0F);
            projection(pc);
            circles[count][0]=iX(pc.x);
            circles[count][2]=(int)((-pc.x) * 2.0F /pixelSize);
            pc.x=0.0F; pc.y= t;
            projection(pc);
            circles[count][1]=iY(pc.y);
            circles[count][3]=(int)(pc.y * 2.0F /pixelSize);
            t += gap;
        }
        if (!isProjected) lastCircleScale=scale;
    }

    /** Create an array of linked lists that point only to the tuples affected by
    *   each particular node, thus allowing faster update of the the locations
    *   and easy display of only the tuples on that column of data, the membership
    *   of these lists is subset of the larger list at 'matrix'
    */
    protected void setColumns()
    {
        int keyWordCount=keyWordArray.getNumWords();
        columns = new Pt2DNode[keyWordCount];
        Pt2DNode tails[] = new Pt2DNode[keyWordCount];
        Pt2DNode k = null;
        for (int i = 0; i< keyWordCount;i++) tails[i] = columns[i]=null;

        if(keyWordCount==0) return;
        Pt2DNode m = matrix;
        SparseNode j=null;

        // This may appear a bit obsure;

        // columns[] is the start of an array of linked lists representing
        // each of the keywords. tails[] is a parallel linked list representing the
        // last added node to the columns[] lists;

        // m loops through the rows and j iterates through the columns of each row
        // when j finds a reference to a particular column[index] row m is at is added to
        // the linked list for that column[index]

        while (m != null)
        {
            j = m.tuple.row;
            while (j != null)
            {
                k = new Pt2DNode(m.tuple);
                int column = j.getColumn();
                if(column > keyWordCount)
                    System.out.println("col "+ column + " of " + keyWordCount);
                else
                {
                    if (tails[column]!=null)
                        tails[column].setNext(k);
                    else columns[column] = k;
                    tails[column]=k;
                }
                j=j.getNext();
            }
            m=m.getNext();
        }
    }
    
    /** Build a linked list of Points and tuples down connected to lists of nodes in the
     * tuples across.
     * @param dataItems the list
     * @param refs[] the document titles
     * @param docCount the number of documents
     * @param wordCount the number of visible words
     */
    protected void setDatalists(SparseMatrix dataItems, String refs[],int docCount, int wordCount)
    {
        Pt2DNode m = null, n=null;
        boolean state;
        for (int co=0;co<docCount;co++)
        {
            SparseNode start = dataItems.getRow(co);
            Point2D pn = new Point2D(0.0F,0.0F);
            state = dataItemXY(pn,start);

            n = new Pt2DNode(pn.x,pn.y,co,start,refs[co]);
            if (state) projection(n.tuple.pnt);
            n.tuple.state =state;
            if (state)
            {
                n.tuple.iPnt.x=iX(n.tuple.pnt.x);
                n.tuple.iPnt.y=iY(n.tuple.pnt.y);
            }
            if (m != null) m.setNext(n);
            else matrix = n;
            m=n;
        }
        setColumns();
    }

    /** Travers the linked lists setting new location values for the document points
     * @param ma a linked list of document nodes
     */
    protected boolean setDocPoints(Pt2DNode ma)
    {
        while (ma != null)
        {
            ma.tuple.state = dataItemXY(ma.tuple.pnt,ma.tuple.row);
            if (ma.tuple.state)
            {
                //c++;
                projection(ma.tuple.pnt);
                ma.tuple.iPnt.x=iX(ma.tuple.pnt.x);
                ma.tuple.iPnt.y=iY(ma.tuple.pnt.y);
                if (ma.tuple.iPnt.x<0 || ma.tuple.iPnt.x>dim.width ||
                    ma.tuple.iPnt.y<0 || ma.tuple.iPnt.y>dim.height)
                    return  true;
            }
            ma=ma.getNext();
        }
        return false;
    }

    /** Fill the integer array spokes[16][2] with the correct numbers for
    *   16 radial points on the outermost circle, by defining a point on
    *  the circle and rotating it by PI/8 radians 16 times
    */
    protected void setSpokes()
    {
        float phi=(float)Math.PI/(float)(spokeCount/2);
        Point2D pt = new Point2D(0.0F,r);
        for (int count=0;count < spokeCount;count++)
        {
            spokes[count][0]=iX(pt.x);
            spokes[count][1]=iY(pt.y);
            rotatePt(pt,phi);
        }
    }

    /** set the key word nodes in an even circle round the rim */
    protected void setKeyNodes()
    {
        int keyWordCount=keyWordArray.getNumEnabled();
        float phi=(float)Math.PI/((float)keyWordCount/2.0F);
        Point2D pt = new Point2D(0.0F,r);
        for (int count=0;count < keyWordCount;count++)
	    {
		points[count] = new Point2D(pt.x,pt.y);
		rotatePt(pt,phi);
	    }
    }
    
    /** find the x y coordinate of a tuple of data
    *   @param theP point to be set
    *   @param tuple linked list of data
    *   @return true if tuple contains a reference to a visible keynode
    */
    protected boolean dataItemXY(Point2D theP, SparseNode tuple)
    {
        int index;
        boolean result=false;
        theP.x=0.0F; theP.y = 0.0F;
        while (tuple != null)
        {
            if (keyWordArray.getState(tuple.getColumn()))
            {
                result = true;
                index = mapping.mapWordsToNodes(tuple.getColumn());
                theP.x += tuple.getValue()/r * points[index].x;
                theP.y += tuple.getValue()/r * points[index].y;
            }
            tuple=tuple.getNext();
        }
        return result;
    }

    /** set the text balloon label at the correct location, to the correct size
    *   and with the correct text
    *   @param theString the text to display
    *   @param x the x coordinate of the mouse
    *   @param y the y coordinate of the mouse
    */
    void setBalloon(String theString,int x,int y)
    {
        Rectangle bounds1 = getBounds();
	
        x-=(bounds1.x)-30; y+=(bounds1.y)+20;
	
        radVizPanel.docLabel.setText(theString);
        int width = balloonFontMetrics.stringWidth(theString)+2;
        int height = balloonFontMetrics.getMaxAscent()+2;
        radVizPanel.docLabel.setBounds(x,y,width,height);
    }

    /**  recalculate edge points and document positions in the light of
    *   of changes to the keywords
    */
    public void recalc()
    {
        mapping = new MapKeyWords(keyWordArray);
        tp =null;
        scale=1.0F;
        tpIndex = 0;
        points= new Point2D[keyWordArray.getNumEnabled()];
        setKeyNodes();
        while (setDocPoints(matrix) && !isProjected) scale*=0.9F ;
        repaint();
    }

    /**
    * constructor sets everything up gets creates an instance of DataLink and
    * gets the data matrix and the keywords calls code to initialise display
    * @param DisplayFrame the central object of the display
    * @param width of canvas
    * @param height of canvas
    * @throws IOException
    */
    public CvManyPoints(Vishnu df) throws IOException
    {
        setVisible(false);
       	this.displayFrame = df;
        this.radVizPanel = displayFrame.radVizPanel;
        this.dataManager = displayFrame.dataManager;
	
       	documentCount=0;
       	int keyWordCount=0;
        keyWordArray = new KeyWordArray();
	
       	initgr();
       	setCircles();
       	setSpokes();
       	/**
       	* add an anonymous class to listen for size changed events
       	*/
	
        addComponentListener(new ComponentAdapter(){
		public void componentResized(ComponentEvent evt)
		{
		    initgr();
		    setCircles();
		    setSpokes();
		    while (setDocPoints(matrix) && !isProjected) scale*=0.9F ;
		    repaint();
		}
	    });
	
        // From Java in a Nutshell
        // listen for mouse button pressed event and if on a node point
        // record the current angle and set the value of tp
        /**
	 * add an anonomous class to listen for mouse pressed events
	 */
        addMouseListener(new MouseAdapter()
	    {
		public void mousePressed(MouseEvent evt)
		{
		    int intX = evt.getX();
		    int intY = evt.getY();
		    float x = fx(intX), y = fy(intY);
		    tp = findPoint(x,y);
		    if (tp == null){
			selectDoc = findDoc(intX, intY);
			if (selectDoc != null){
			    StringBuffer buf = new StringBuffer("Doc: ");
			    buf.append(selectDoc.tuple.ref);
			    buf.append("  ");
			    if (buf.length() > 30){
				buf.setLength(30);
				buf.append("... ");
			    }
			    String theString = new String(buf);
			    //theOuterFrame.setDocSelectMenu(theString,selectDoc.tuple.index);
			}   
			//else theOuterFrame.setDocSelectMenu("No Document Selected      ",-1);
			repaint();
			return;
		    }
		    lastAngle=(float)Math.atan2(x,y);
		    repaint();
		}
		
		public void mouseClicked(MouseEvent event)
		{
		    int intX = event.getX();
		    int intY = event.getY();
		    
		    selectDoc = findDoc(intX, intY);
		    if (selectDoc != null){
			if(event.getClickCount()>1){
			    StringBuffer buf = new StringBuffer("Doc: ");
			    buf.append(selectDoc.tuple.ref);
			    buf.append("  ");
			    if (buf.length() > 30){
				buf.setLength(30);
				buf.append("... ");
			    }
			    String theString = new String(buf);
			    //theOuterFrame.setDocSelectMenu(theString,selectDoc.tuple.index);
			    dataManager.fetch(selectDoc.tuple.index);
			}
		    }
		}
		
		public void mouseReleased(MouseEvent event)
		{
		    Cursor releaseCursor = new Cursor(Cursor.DEFAULT_CURSOR);
		    displayFrame.radVizPanel.setCursor(releaseCursor);
		}
	    });
	
        // listen for mouse button dragged event and if tp is set
        // work out a new position for tp and the data points
        /**
	 * add an anonomous class to listen for mouse motion events
	 */
        addMouseMotionListener(new MouseMotionAdapter(){
		public void mouseDragged(MouseEvent evt)
		{
		    Cursor dragCursor = new Cursor(Cursor.HAND_CURSOR);
		    displayFrame.radVizPanel.setCursor(dragCursor);
		    
		    float x = fx(evt.getX()), y = fy(evt.getY());
		    if (tp!=null){
			float thisAngle = (float)Math.atan2(x,y);
			if(isLocked){
			    float phi = lastAngle-thisAngle;
			    rotatePt(tp, phi);
			}
			else{
			    tp.x = x;
			    tp.y = y;
			}
			if (columns!= null){
			    int index = mapping.mapNodesToWords(tpIndex);
			    if (columns[index]!= null){
				if(setDocPoints( columns[index] ) && !isProjected){
				    do{
					scale*=0.9;
				    } while(setDocPoints( matrix ));
				}
				repaint();
			    }
			}
			lastAngle=thisAngle;
		    }
		}
		
		
		
		/** listen for mouse movement and try and find a document
		 * report its presence with a balloon label or ensure
		 * the balloon label is hidden
		 * @param evt the mouse event
		 */
		public void mouseMoved(MouseEvent evt)
		{
		    System.out.println("Moving");
		    /* int intX = evt.getX();
		    int intY = evt.getY();
		    Pt2DNode theNode = findDoc(intX, intY);
		    
		    if (theNode != null){
			if (atDoc) radVizPanel.docLabel.setVisible(false);
			String theString = new String("Doc: " + theNode.tuple.ref);
			setBalloon(theNode.tuple.ref,intX,intY);
			radVizPanel.docLabel.setVisible(true);
			atDoc = true;
		    }
		    else{
			if (atDoc){
			    radVizPanel.docLabel.setVisible(false);
			}
			atDoc = false;
			}*/
		    return;
		}
	    });
    }

    /* the Data loading happens in here now */
    
    public void update()
    {
        int maxWordsCount;
        this.radVizPanel = displayFrame.radVizPanel;
        this.dataManager = displayFrame.dataManager;

        //if(dataManager.getDocCount()==0)return;
        selectDoc = null;
        tp =null;
       	docs = dataManager.getDocs();
       	documentCount = docs.length;
        String [] words = dataManager.getWords();
        maxWordsCount = words.length;
        //maxWordsCount= in.setKeywordCount(maxWordsCount);
        int keyWordCount = Math.min(maxWordsCount,12);
       	keyWordArray = new KeyWordArray(words,words.length);
        maxWordsCount = keyWordArray.getNumWords();


       	mapping = new MapKeyWords(keyWordArray);

        String titles[] = dataManager.getDescriptions();
        if(titles==null) System.out.println("no descriptions read in");
        points= new Point2D[keyWordCount];
        balloonFont = new Font("Dialog", Font.PLAIN, 16);
        balloonFontMetrics = getFontMetrics(balloonFont);
        theDataMatrix = dataManager.getMatrix();
        fullMatrix = theDataMatrix.getExpanded();
        setKeyNodes();
        setDatalists(theDataMatrix,titles,documentCount,maxWordsCount);
        recalc();
    }
    /** Cluster the words round the edge
    * @param type the type of clustering
    */
    public void cluster()
    {
        if(documentCount ==0) return;
        int keyWordCount=keyWordArray.getNumEnabled();
        int keylist[] = new int[keyWordCount];

        for (int c=0;c<keylist.length;c++)
        {
            keylist[c]=mapping.mapNodesToWords(c);
        }
        if(fullMatrix==null) System.out.println("full matrix is null");
        else
        {
            if(fullMatrix[0]==null) System.out.println("full matrix[0] is null");
        }

        keylist = HitCluster.maximunDistance (fullMatrix, keylist, documentCount);

        mapping.reMapKeyWords(keylist,keyWordArray);

        tp =null;
        tpIndex = 0;
        points= new Point2D[keyWordArray.getNumEnabled()];
        setKeyNodes();
        while (setDocPoints(matrix) && !isProjected) scale *=0.9;
        repaint();
    }
    /**
    * Overides setSize so that the drawing is correctly scaled on a setSize call
    * @param new dimensions
    */
    public void setSize(Dimension d)
    {
        super.setSize(d);
        initgr();
        setCircles();
        setSpokes();
        while (setDocPoints(matrix) && !isProjected) scale*=0.9F ;
    }
    /**
    * Overides setSize so that the drawing is correctly scaled on a setSize call
    * @param new width
    * @param new height
    */
    public void setSize(int x, int y)
    {
        Dimension d = new Dimension(x,y);
        setSize(d);
    }



    /**
    * Tidy up the image before canvas is destroyed a bit like a destructor in C
    */
    public void dispose()
    {
        gImage.dispose();
    }


    /** Set the shape of the edge word nodes
    * @param state selects the shape
    */
    void setNodeShape(int state)
    {
        nodesShape = state;
        repaint();
    }

    /** Set the shape of the centre document nodes
    *  @param state selects the shape
    */
    void setDocShape(int state)
    {
        docsShape = state;
        repaint();
    }

    /** switches the projection on and off
    * @param state is the switch
    */
    public void projection(boolean state)
    {
        // to do: code goes here.
        isProjected = state;
        circleCount = state?40:4;
        setCircles();
        while (setDocPoints(matrix) && !isProjected) scale*=0.9;
        repaint();
    }

    /**
    * Display or not the grid
    * @param switches on and off
    */

    public void grid(boolean state)
    {
        displayGrid=state;
        repaint();
    }

    /**
    * Display or not the rim
    * @param switches on and off
    */
    public void rim(boolean state)
    {
        displayRim=state;
        repaint();
    }

    /**
    * Lock or not the edge nodes so they can only be moved in a circle
    * @param switches on and off
    */
    public void lock(boolean state)
    {
        isLocked = state;
    }


    /**
    * Flattens the projection slightly
    *
    */
    public void flatter()
    {
        if(!isProjected)
            scale/=0.9;
        else
        {
            factor *=0.9;
            setCircles();
        }
        while (setDocPoints(matrix)&& !isProjected) scale*=0.9;
        repaint();
    }

    /**
    * Heightens the projection slightly
    *
    */
    public void higher()
    {
        if(!isProjected)
            scale*=0.9;
        else
        {
            factor /=0.9;
            setCircles();
        }
        while (setDocPoints(matrix) && !isProjected) scale*=0.9;
        repaint();
    }




    /**
    * Translate to integer raster coordinate X from
    * floating point coord X based around a notional centre
    * @param x floating point x coordinate
    * @return integer X coordinate
    */
    int iX(float x){return Math.round(centerX + x/pixelSize);}
    /**
    * Translate to integer raster coordinate Y from
    * floating point coord Y based around a notional centre
    * @param y floating point y coordinate
    * @return integer Y coordinate
    */
    int iY(float y){return Math.round(centerY - y/pixelSize);}
    /**
    * Translate from integer raster coordinate X to
    * floating point coord X based around a notional centre
    * @param x integer x coordinate
    * @return floating point X coordinate
    */
    float fx(int X){return (X - centerX) * pixelSize;}
    /**
    * Translate from integer raster coordinate Y to
    * floating point coord Y based around a notional centre
    * @param y integer y coordinate
    * @return floating point Y coordinate
    */
    float fy(int Y){return (centerY - Y) * pixelSize;}

    /**
    * find if an edge keyword node is at the x y location
    * @param x fp coordinate
    * @param y fp coordinate
    * @return fp Point at location or null
    */
    public Point2D findPoint(float x, float y)
    {
        int keyWordCount=keyWordArray.getNumEnabled();
        for (int i=0;i<keyWordCount;i++)
        {
            int intX = iX(x), intY = iY(y),
                lX = iX(points[i].x)-3, hX = lX+6,
                lY = iY(points[i].y)-3, hY = lY+6;

            if (intX > lX && intX < hX && intY > lY && intY < hY)
            {

                tpIndex = i;
                return points[i];
            }
        }
        return null;
    }

    /**
    * find if a Doc node is at the x y location
    * @param x int coordinate
    * @param y int coordinate
    * @return fp Point at location or null
    */
    public Pt2DNode findDoc(int intX, int intY)
    {
        Pt2DNode thePnt=matrix;
        while (thePnt!= null)
        {
            if (thePnt.tuple.state)
            {
                int   lX = thePnt.tuple.iPnt.x-3, hX = lX+6,
                    lY = thePnt.tuple.iPnt.y-3, hY = lY+6;
                if (intX > lX && intX < hX && intY > lY && intY < hY) return thePnt;
            }
            thePnt = thePnt.getNext();
        }
        return null;
    }
    /** project the point radially from the centre using the arctan function
    *  if isProjected is true or scale them it false
    *  @param point to be projected
    *  @param ingore scaling flag
    */
    protected void projection(Point2D p)
    {
        if (!isProjected)
        {
            if(scale!=1.0)
            {
                p.x*=scale;
                p.y*=scale;
            }
            return;
        }
        float newRad;
        float rad = (float)Math.sqrt(p.x*p.x+p.y*p.y);
        float sine = p.y/rad;
        float cosine = p.x/rad;
        newRad =(float)Math.atan(rad/(factor*Math.PI))*magic;
        p.x = newRad * cosine;
        p.y = newRad * sine;
    }

    /** set up graphic settings such as centre and pixelsize
    */
    protected void initgr()
    {
        dim = getSize();
        int maxX = dim.width - 1, maxY = dim.height - 1;
        pixelSize = Math.max(rWidth/maxX, rHeight/maxY);
        centerX = maxX/2; centerY = maxY/2;
    }


    /////////Athina has been here!!!!!///////////////////////////////////////

    /** draw a for document node shape at raster x y
    *   @param x location
    *   @param y location
    *   @param g Graphics object
    */
    protected void drawItemPoint(int x, int y, Graphics g)
    {

        if (docsShape == SQUARESHAPE)
            oval(g,x,y);
        else if (docsShape == ROUNDSHAPE)
            rect(g,x,y);
        else
            diamond(g,x,y);

    }

    /**
    *   draw a for keyword node shape at raster x y
    *   @param x location
    *   @param y location
    *   @param g Graphics object
    */
    void drawNodes(int x, int y, Graphics g)
    {
        if (nodesShape == 0)
            oval(g,x,y);
        else if (nodesShape == 1)
            rect(g,x,y);
        else
            diamond(g,x,y);

    }


    /**
    *  draw the key name near the edgenode
    *  @param g Graphics class
    *  @param count index of the name to draw
    *  @param select if the node is selected.
    */
    void drawKeyName(Graphics g, int count, boolean select)
    {
        FontMetrics fontMetrics = g.getFontMetrics();
        Font font = new Font("arial", Font.PLAIN, 16);
        String word = keyWordArray.getWord(mapping.mapNodesToWords(count));
        int posX= iX(points[count].x)-2, posY= iY(points[count].y)-2;
        int width = fontMetrics.stringWidth(word);
        if (points[count].x < 0.0)
            posX -= width - (int) (points[count].x/2.00);
        else
            posX += (int) (points[count].x/2.00);

        if (points[count].y < 0.0)
            posY += fontMetrics.getMaxAscent()- (int)(points[count].y/2.00);
        else
            posY -= (int) (points[count].y/2.00);
        g.setFont(font);
        g.drawString(word,posX,posY);
        if (select){ g.drawRect(posX-2,posY-15,width+4,20);}
    }

    /**
    * draw a box at the edge nodes
    * @param g Graphics object
    */
    protected void drawEdgeNodes(Graphics g)
    {
        int keyWordCount=keyWordArray.getNumEnabled();

        for (int count=0;count < keyWordCount; count++)
        {
            drawNodes(iX(points[count].x), iY(points[count].y), g);
            drawKeyName(g,count,false);

        }
    }
    /**
    * Draw the documents that have the selected key word in them
    * @param g Graphics object
    */
    protected void drawSelect(Graphics g)
    {
        if (tp!=null)
        {
            int x = iX(points[tpIndex].x), y = iY(points[tpIndex].y);
            g.setColor(defSelect);
            drawNodes(x, y, g);

            Pt2DNode m = columns[mapping.mapNodesToWords(tpIndex)];
            while (m != null)
            {
                x=m.tuple.iPnt.x;
                y=m.tuple.iPnt.y;

                drawItemPoint(x, y, g);

                m=m.getNext();
            }
            g.setColor(defDocsColor);

        }
    }
    /**
    * draw the key word nodes that feature in the selected document
    */
    protected void drawDocSelect(Graphics g)
    {
        if (selectDoc==null) return;
        g.setColor(defHighlight);
        int x =selectDoc.tuple.iPnt.x, y =selectDoc.tuple.iPnt.y;
        drawItemPoint(x, y, g);

        SparseNode fn = selectDoc.tuple.row;
        while (fn != null)
        {
            if (keyWordArray.getState(fn.getColumn()))
            {
                int index = mapping.mapWordsToNodes(fn.getColumn());
                drawNodes(iX(points[index].x), iY(points[index].y), g);
                drawKeyName(g,index,true);
            }
            fn=fn.getNext();

        }
        g.setColor(defNodesColor);

    }

    /**
    * draw all the document crosses
    * @param g Graphics object
    */

    protected void drawDocXs(Graphics g)
    {
        Pt2DNode m = matrix;
        while (m != null)
        {
            if (m.tuple.state) drawItemPoint(m.tuple.iPnt.x, m.tuple.iPnt.y, g);
            m=m.getNext();
        }
    }

    /**
    * draw the target
    * @param g Graphics object
    */

    protected void drawTarget(Graphics g)
    {
        if (!isProjected && lastCircleScale!=scale)
            setCircles();

        if (displayRim)
        {
            g.setColor(Color.darkGray);
            g.drawOval(rim[0],rim[1],rim[2],rim[3]);
        }
        if (displayGrid)
        {
            g.setColor(Color.lightGray);
            for (int count=0;count < circleCount; count++)
                g.drawOval(circles[count][0],circles[count][1],circles[count][2],circles[count][3]);
            for (int count=0;count < spokeCount; count++)
                g.drawLine(centerX, centerY,spokes[count][0],spokes[count][1]);
        }
    }

    /**
    * Repaint the screen overides canvas.repaint()
    */

    public void repaint()
    {
        isDirty=true;
        super.repaint();
    }

    /**
    * Repaint the screen overides canvas.update()
    * to stop flickering
    * @param g Graphics object
    */

    public void update(Graphics g)
    {
        paint(g);
    }

    /**
    * draw a filled square with a centre at x y
    * @param g Graphics object
    * @param x location
    * @param y location
    */

    public void rect(Graphics g, int x, int y)
    {
        g.drawRect(x-2,y-2,4,4);
        g.fillRect(x-2,y-2,4,4);
        repaint();
    }

    /**
    * draw a filled circle with a centre at x y
    * @param g Graphics object
    * @param x location
    * @param y location
    */
    public void oval(Graphics g, int x, int y)
    {
        g.drawOval(x-2,y-2,4,4);
        g.fillOval(x-2,y-2,4,4);
        repaint();
    }

    /**
    * draw a filled diamond with a centre at x y
    * @param g Graphics object
    * @param x location
    * @param y location
    */
    public void diamond(Graphics g, int x, int y)
    {
        int numpoints = 4;
        int[] xpoints = new int[numpoints+1];
        int[] ypoints = new int[numpoints+1];
        for(int i=0; i < numpoints; i++) {
            double angle = 2*Math.PI * i / numpoints;
            xpoints[i] = (int)(x + 3*Math.cos(angle));
            ypoints[i] = (int)(y - 3*Math.sin(angle));
        }

        g.fillPolygon(xpoints, ypoints, numpoints);
        xpoints[numpoints] = xpoints[0];  ypoints[numpoints] = ypoints[0];
        g.drawPolygon(xpoints, ypoints, numpoints+1);
        repaint();
    }

    /**
    * paint the whole thing
    * @param g Graphics object
    */

    public void paint(Graphics g)
    {
	System.out.println("Repaint CvManyPoints");
        if(!isVisible()) return;
        if (isDirty==true)
        {
            if (imgW != dim.width || imgH != dim.height)
            {
                if( gImage != null ) gImage.dispose();
                imgW = dim.width; imgH = dim.height;
                image = createImage(imgW, imgH);
                gImage = image.getGraphics();
            }
            gImage.setColor(Color.white);
            gImage.fillRect(0, 0, imgW, imgH);

            drawTarget(gImage);
            gImage.setColor(defNodesColor);
            drawEdgeNodes(gImage);
            gImage.setColor(defDocsColor);
            drawDocXs(gImage);

            if (tp!=null)
                drawSelect(gImage);
            if (selectDoc!=null)
                drawDocSelect(gImage);

            //gImage.drawRect(1,1,dim.width-1,dim.height-1);
        }
        g.drawImage(image, 0, 0, null);

        isDirty=false;
    }
}







